# -*- #################
# ---------------------------------------------------------------------------
# normalization_min-max.py
# 2019 Andrzej Świąder
# AGH University of Science and Technology, Kraków, Poland
# Description:
# ---------------------------------------------------------------------------

# Import arcpy module
import arcpy, os
from arcpy.sa import *

# Script arguments
inputRas = arcpy.GetParameterAsText(0)
normMinIn = arcpy.GetParameterAsText(1)
normMaxIn = arcpy.GetParameterAsText(2)
zeroMidSym = arcpy.GetParameterAsText(3)
zeroMidAsym = arcpy.GetParameterAsText(4)

normMin = float(normMinIn)
normMax = float(normMaxIn)

ptName = inputRas.split('.') # preparing filename for result file

# Set Geoprocessing environments
arcpy.env.compression = "LZ77"
arcpy.env.rasterStatistics = "STATISTICS 1 1"
arcpy.env.resamplingMethod = "BILINEAR"
arcpy.env.tileSize = "128 128"
arcpy.env.pyramid = "PYRAMIDS -1 BILINEAR DEFAULT 75 NO_SKIP"
arcpy.env.nodata = "NONE"

# Get input raster min and max values
arcpy.CalculateStatistics_management(inputRas, "", "", "", "SKIP_EXISTING", "")

outMin = arcpy.GetRasterProperties_management(inputRas, 'MINIMUM')
rasMin = float(outMin.getOutput(0).replace(',','.'))

# ".replace(',','.')" above and below was used because of problems with internalization. Many thanks to xander_bakker for the solution (https://community.esri.com/thread/170917)

outMax = arcpy.GetRasterProperties_management(inputRas, 'MAXIMUM')
rasMax = float(outMax.getOutput(0).replace(',','.'))

#arcpy.AddMessage("rasMin: {0}".format(rasMin))
#arcpy.AddMessage("rasMax: {0}".format(rasMax))

inputRaster = Raster(inputRas)

if zeroMidSym == 'false' and zeroMidAsym == 'false':
	outRasNorm = ptName[0] + '_norm_{0}_{1}.tif'.format(normMinIn, normMaxIn) # preparing filename for result file

	outRasNormalized = ((inputRaster - rasMin) / (rasMax - rasMin)) * (normMax - normMin) + normMin

if zeroMidSym == 'true':
	outRasNorm = ptName[0] + '_norm_{0}_{1}_zeroAsMidSym.tif'.format(normMinIn, normMaxIn) # preparing filename for result file
	
	#arcpy.AddMessage("rasMin: {0}".format(rasMin))
	#arcpy.AddMessage("rasMax: {0}".format(rasMax))
	
	outRasNormalized = Con(inputRaster < 0, ((inputRaster - rasMin) / (0 - rasMin)) * (0 - normMin) + normMin, ((inputRaster - 0) / (rasMax - 0)) * (normMax - 0) + 0)
	
	
if zeroMidAsym == 'true':
	outRasNorm = ptName[0] + '_norm_{0}_{1}_zeroAsMidAsym.tif'.format(normMinIn, normMaxIn) # preparing filename for result file
	
	if abs(rasMin) < rasMax:
		rasMin = -rasMax
	
	elif rasMax < abs(rasMin):
		rasMax = -rasMin
		
	#arcpy.AddMessage("rasMin: {0}".format(rasMin))
	#arcpy.AddMessage("rasMax: {0}".format(rasMax))
	
	outRasNormalized = Con(inputRaster < 0, ((inputRaster - rasMin) / (0 - rasMin)) * (0 - normMin) + normMin, ((inputRaster - 0) / (rasMax - 0)) * (normMax - 0) + 0)
	
if os.path.isfile(outRasNorm): # check if result file already exist; if yes - skip and move to the next file from list
	arcpy.AddWarning("\nFile {0} already exist - skipping".format(outRasNorm))
else:
	arcpy.CopyRaster_management(outRasNormalized, outRasNorm, format="TIFF")
